@@ -7,13 +7,13 @@ import numpy as np
 import pyarrow.parquet as pq
 from collections import namedtuple
 from embedding_reader.get_file_list import get_file_list
-from embedding_reader.piece_builder import build_pieces
+from embedding_reader.piece_builder import build_pieces, PIECES_BASE_COLUMNS
 from threading import Semaphore
 import math
 
 
 class ParquetReader:
-    """Parquet reader class, implements init to read the files headers and call to procuce embeddings batches"""
+    """Parquet reader class, implements init to read the files headers and call to produce embeddings batches"""
 
     def __init__(self, embeddings_folder, embedding_column_name, metadata_column_names=None):
         self.embeddings_folder = embeddings_folder
@@ -35,14 +35,19 @@ class ParquetReader:
                     continue
 
         def file_to_header(filename):
-            with self.fs.open(filename, "rb") as f:
-                parquet_file = pq.ParquetFile(f, memory_map=True)
-                return [filename, parquet_file.metadata.num_rows]
+            try:
+                with self.fs.open(filename, "rb") as f:
+                    parquet_file = pq.ParquetFile(f, memory_map=True)
+                    return (None, [filename, parquet_file.metadata.num_rows])
+            except Exception as e:  # pylint: disable=broad-except
+                return e, (filename, None)
 
         headers = []
         count_before = 0
         with ThreadPool(10) as p:
-            for c in tqdm(p.imap(file_to_header, embeddings_file_paths), total=len(embeddings_file_paths)):
+            for err, c in tqdm(p.imap(file_to_header, embeddings_file_paths), total=len(embeddings_file_paths)):
+                if err is not None:
+                    raise Exception(f"failed reading file {c[0]}") from err
                 if c[1] == 0:
                     continue
                 headers.append([*c, count_before])
@@ -58,7 +63,7 @@ class ParquetReader:
 
     def __call__(self, batch_size, start=0, end=None, max_piece_size=None, parallel_pieces=None, show_progress=True):
         if end is None:
-            end = self.headers["count"].sum()
+            end = self.count
 
         if end > self.count:
             end = self.count
@@ -73,39 +78,27 @@ class ParquetReader:
         pieces = build_pieces(
             headers=self.headers, batch_size=batch_size, start=start, end=end, max_piece_size=max_piece_size
         )
-        batch_count = pieces["batch_id"].max() + 1
 
-        cols = [
-            "filename",
-            "piece_start",
-            "piece_end",
-            "piece_length",
-            "batch_id",
-            "batch_start",
-            "batch_end",
-            "batch_length",
-            "last_piece",
-        ]
+        cols = PIECES_BASE_COLUMNS
         Piece = namedtuple("Count", cols)
 
         def read_piece(piece):
-            start = piece.piece_start
-            end = piece.piece_end
-            path = piece.filename
+            try:
+                start = piece.piece_start
+                end = piece.piece_end
+                path = piece.filename
 
-            with self.fs.open(path, "rb") as f:
-                length = end - start
-                table = pq.read_table(f)
-                id_columns = self.metadata_column_names
-                table_slice = table.slice(start, length)
-                embeddings_raw = table_slice[self.embedding_column_name].to_numpy()
-                ids = table_slice.select(id_columns).to_pandas() if id_columns else None
+                with self.fs.open(path, "rb") as f:
+                    length = end - start
+                    table = pq.read_table(f)
+                    id_columns = self.metadata_column_names
+                    table_slice = table.slice(start, length)
+                    embeddings_raw = table_slice[self.embedding_column_name].to_numpy()
+                    ids = table_slice.select(id_columns).to_pandas() if id_columns else None
 
-                return (
-                    np.stack(embeddings_raw),
-                    ids,
-                    piece,
-                )
+                    return (None, (np.stack(embeddings_raw), ids, piece,))
+            except Exception as e:  # pylint: disable=broad-except
+                return e, (None, None, piece)
 
         semaphore = Semaphore(parallel_pieces)
 
@@ -119,9 +112,14 @@ class ParquetReader:
         batch_offset = 0
 
         if show_progress:
-            pbar = tqdm(total=batch_count)
+            pbar = tqdm(total=len(pieces))
         with ThreadPool(parallel_pieces) as p:
-            for data, meta, piece in p.imap(read_piece, piece_generator(pieces)):
+            for err, (data, meta, piece) in p.imap(read_piece, piece_generator(pieces)):
+                if err is not None:
+                    semaphore.release()
+                    raise Exception(
+                        f"failed reading file {piece.filename} from {piece.piece_start} to {piece.piece_end}"
+                    ) from err
                 if batch is None:
                     batch = np.empty((piece.batch_length, self.dimension), "float32")
                     if self.metadata_column_names is not None:
@@ -143,8 +141,9 @@ class ParquetReader:
                     if self.metadata_column_names is not None:
                         batch_meta = None
                     batch_offset = 0
-                    if show_progress:
-                        pbar.update(1)
+
+                if show_progress:
+                    pbar.update(1)
                 semaphore.release()
 
         if show_progress:
